@@ -8,14 +8,19 @@
 
 typedef struct file_buffer buf_T; // Forward declaration
 
+// Reference to a buffer that stores the value of buf_free_count.
+// bufref_valid() only needs to check "buf" when the count differs.
+typedef struct {
+  buf_T *br_buf;
+  int    br_buf_free_count;
+} bufref_T;
+
 // for garray_T
 #include "nvim/garray.h"
 // for pos_T, lpos_T and linenr_T
 #include "nvim/pos.h"
 // for the number window-local and buffer-local options
 #include "nvim/option_defs.h"
-// for optional iconv support
-#include "nvim/iconv.h"
 // for jump list and tag stack sizes in a buffer and mark types
 #include "nvim/mark_defs.h"
 // for u_header_T; needs buf_T.
@@ -23,7 +28,7 @@ typedef struct file_buffer buf_T; // Forward declaration
 // for hashtab_T
 #include "nvim/hashtab.h"
 // for dict_T
-#include "nvim/eval_defs.h"
+#include "nvim/eval/typval.h"
 // for proftime_T
 #include "nvim/profile.h"
 // for String
@@ -31,7 +36,7 @@ typedef struct file_buffer buf_T; // Forward declaration
 // for Map(K, V)
 #include "nvim/map.h"
 
-#define MODIFIABLE(buf) (!buf->terminal && buf->b_p_ma)
+#define MODIFIABLE(buf) (buf->b_p_ma)
 
 /*
  * Flags for w_valid.
@@ -83,36 +88,25 @@ typedef struct file_buffer buf_T; // Forward declaration
 typedef struct window_S win_T;
 typedef struct wininfo_S wininfo_T;
 typedef struct frame_S frame_T;
-typedef int scid_T;                     /* script ID */
 
 // for struct memline (it needs memfile_T)
 #include "nvim/memline_defs.h"
-
 // for struct memfile, bhdr_T, blocknr_T... (it needs buf_T)
 #include "nvim/memfile_defs.h"
 
-/*
- * This is here because regexp_defs.h needs win_T and buf_T. regprog_T is
- * used below.
- */
+// for regprog_T. Needs win_T and buf_T.
 #include "nvim/regexp_defs.h"
-
-// for  synstate_T (needs reg_extmatch_T, win_T and buf_T)
+// for synstate_T (needs reg_extmatch_T, win_T, buf_T)
 #include "nvim/syntax_defs.h"
-
 // for signlist_T
 #include "nvim/sign_defs.h"
-
 // for bufhl_*_T
 #include "nvim/bufhl_defs.h"
 
 typedef Map(linenr_T, bufhl_vec_T) bufhl_info_T;
 
-// for FileID
-#include "nvim/os/fs_defs.h"
-
-// for Terminal
-#include "nvim/terminal.h"
+#include "nvim/os/fs_defs.h"    // for FileID
+#include "nvim/terminal.h"      // for Terminal
 
 /*
  * The taggy struct is used to store the information about a :tag command.
@@ -237,6 +231,8 @@ typedef struct {
 # define w_p_crb w_onebuf_opt.wo_crb    /* 'cursorbind' */
   int wo_crb_save;              /* 'cursorbind' state saved for diff mode*/
 # define w_p_crb_save w_onebuf_opt.wo_crb_save
+  char_u *wo_scl;
+# define w_p_scl w_onebuf_opt.wo_scl    // 'signcolumn'
 
   int wo_scriptID[WV_COUNT];            /* SIDs for window-local options */
 # define w_p_scriptID w_onebuf_opt.wo_scriptID
@@ -322,25 +318,6 @@ typedef struct {
   String save_inputbuf;
 } tasave_T;
 
-/*
- * Used for conversion of terminal I/O and script files.
- */
-typedef struct {
-  int vc_type;                  /* zero or one of the CONV_ values */
-  int vc_factor;                /* max. expansion factor */
-# ifdef USE_ICONV
-  iconv_t vc_fd;                /* for CONV_ICONV */
-# endif
-  bool vc_fail;                 /* fail for invalid char, don't use '?' */
-} vimconv_T;
-
-#define CONV_NONE               0
-#define CONV_TO_UTF8            1
-#define CONV_9_TO_UTF8          2
-#define CONV_TO_LATIN1          3
-#define CONV_TO_LATIN9          4
-#define CONV_ICONV              5
-
 /*
  * Structure used for mappings and abbreviations.
  */
@@ -451,6 +428,13 @@ typedef struct {
   char_u *b_syn_isk;            // iskeyword option
 } synblock_T;
 
+/// Type used for changedtick_di member in buf_T
+///
+/// Primary exists so that literals of relevant type can be made.
+typedef TV_DICTITEM_STRUCT(sizeof("changedtick")) ChangedtickDictItem;
+
+#define BUF_HAS_QF_ENTRY 1
+#define BUF_HAS_LL_ENTRY 2
 
 /*
  * buffer: structure that holds information about one file
@@ -471,9 +455,9 @@ struct file_buffer {
 
   int b_nwindows;               /* nr of windows open on this buffer */
 
-  int b_flags;                  /* various BF_ flags */
-  bool b_closing;               /* buffer is being closed, don't let
-                                   autocommands close it too. */
+  int b_flags;                  // various BF_ flags
+  int b_locked;                 // Buffer is being closed or referenced, don't
+                                // let autocommands wipe it out.
 
   /*
    * b_ffname has the full path of the file (NULL for no name).
@@ -490,7 +474,9 @@ struct file_buffer {
 
   int b_changed;                // 'modified': Set to true if something in the
                                 // file has been changed and not written out.
-  int b_changedtick;            // incremented for each change, also for undo
+/// Change identifier incremented for each change, including undo
+#define b_changedtick changedtick_di.di_tv.vval.v_number
+  ChangedtickDictItem changedtick_di;  // b:changedtick dictionary item.
 
   bool b_saving;                /* Set to true if we are in the middle of
                                    saving the buffer. */
@@ -609,6 +595,7 @@ struct file_buffer {
   int b_p_bomb;                 ///< 'bomb'
   char_u *b_p_bh;               ///< 'bufhidden'
   char_u *b_p_bt;               ///< 'buftype'
+  int b_has_qf_entry;           ///< quickfix exists for buffer
   int b_p_bl;                   ///< 'buflisted'
   int b_p_cin;                  ///< 'cindent'
   char_u *b_p_cino;             ///< 'cinoptions'
@@ -638,6 +625,7 @@ struct file_buffer {
   char_u *b_p_inde;             ///< 'indentexpr'
   uint32_t b_p_inde_flags;      ///< flags for 'indentexpr'
   char_u *b_p_indk;             ///< 'indentkeys'
+  char_u *b_p_fp;               ///< 'formatprg'
   char_u *b_p_fex;              ///< 'formatexpr'
   uint32_t b_p_fex_flags;       ///< flags for 'formatexpr'
   char_u *b_p_kp;               ///< 'keywordprg'
@@ -651,6 +639,7 @@ struct file_buffer {
   char_u *b_p_qe;               ///< 'quoteescape'
   int b_p_ro;                   ///< 'readonly'
   long b_p_sw;                  ///< 'shiftwidth'
+  long b_p_scbk;                ///< 'scrollback'
   int b_p_si;                   ///< 'smartindent'
   long b_p_sts;                 ///< 'softtabstop'
   long b_p_sts_nopaste;         ///< b_p_sts saved for paste mode
@@ -731,8 +720,8 @@ struct file_buffer {
   int b_bad_char;               /* "++bad=" argument when edit started or 0 */
   int b_start_bomb;             /* 'bomb' when it was read */
 
-  dictitem_T b_bufvar;          /* variable for "b:" Dictionary */
-  dict_T      *b_vars;          /* internal variables, local to buffer */
+  ScopeDictDictItem b_bufvar;  ///< Variable for "b:" Dictionary.
+  dict_T *b_vars;  ///< b: scope dictionary.
 
   /* When a buffer is created, it starts without a swap file.  b_may_swap is
    * then set to indicate that a swap file may be opened later.  It is reset
@@ -820,10 +809,9 @@ struct tabpage_S {
   buf_T           *(tp_diffbuf[DB_COUNT]);
   int tp_diff_invalid;              ///< list of diffs is outdated
   frame_T         *(tp_snapshot[SNAP_COUNT]);    ///< window layout snapshots
-  dictitem_T tp_winvar;             ///< variable for "t:" Dictionary
-  dict_T          *tp_vars;         ///< internal variables, local to tab page
-  char_u          *localdir;        ///< Absolute path of local directory or
-                                    ///< NULL
+  ScopeDictDictItem tp_winvar;      ///< Variable for "t:" Dictionary.
+  dict_T          *tp_vars;         ///< Internal variables, local to tab page.
+  char_u          *tp_localdir;     ///< Absolute path of local cwd or NULL.
 };
 
 /*
@@ -877,16 +865,17 @@ struct frame_S {
  * match functions there is a different pattern for each window.
  */
 typedef struct {
-  regmmatch_T rm;       /* points to the regexp program; contains last found
-                           match (may continue in next line) */
-  buf_T       *buf;     /* the buffer to search for a match */
-  linenr_T lnum;        /* the line to search for a match */
-  int attr;             /* attributes to be used for a match */
-  int attr_cur;           /* attributes currently active in win_line() */
-  linenr_T first_lnum;          /* first lnum to search for multi-line pat */
-  colnr_T startcol;       /* in win_line() points to char where HL starts */
-  colnr_T endcol;        /* in win_line() points to char where HL ends */
-  proftime_T tm;        /* for a time limit */
+  regmmatch_T rm;       // points to the regexp program; contains last found
+                        // match (may continue in next line)
+  buf_T       *buf;     // the buffer to search for a match
+  linenr_T lnum;        // the line to search for a match
+  int attr;             // attributes to be used for a match
+  int attr_cur;         // attributes currently active in win_line()
+  linenr_T first_lnum;  // first lnum to search for multi-line pat
+  colnr_T startcol;     // in win_line() points to char where HL starts
+  colnr_T endcol;       // in win_line() points to char where HL ends
+  bool is_addpos;       // position specified directly by matchaddpos()
+  proftime_T tm;        // for a time limit
 } match_T;
 
 /// number of positions supported by matchaddpos()
@@ -1035,11 +1024,11 @@ struct window_S {
    */
   int w_wrow, w_wcol;               /* cursor position in window */
 
-  linenr_T w_botline;               /* number of the line below the bottom of
-                                       the screen */
-  int w_empty_rows;                 /* number of ~ rows in window */
-  int w_filler_rows;                /* number of filler rows at the end of the
-                                       window */
+  linenr_T w_botline;               // number of the line below the bottom of
+                                    // the window
+  int w_empty_rows;                 // number of ~ rows in window
+  int w_filler_rows;                // number of filler rows at the end of the
+                                    // window
 
   /*
    * Info about the lines currently in the window is remembered to avoid
@@ -1114,8 +1103,8 @@ struct window_S {
 
   long w_scbind_pos;
 
-  dictitem_T w_winvar;          /* variable for "w:" Dictionary */
-  dict_T      *w_vars;          /* internal variables, local to window */
+  ScopeDictDictItem w_winvar;  ///< Variable for "w:" dictionary.
+  dict_T *w_vars;  ///< Dictionary with w: variables.
 
   int w_farsi;                  /* for the window dependent Farsi functions */
 
