@@ -1,3 +1,6 @@
+// This is an open source non-commercial project. Dear PVS-Studio, please check
+// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
+
 #include <stdbool.h>
 #include <inttypes.h>
 
@@ -12,6 +15,7 @@
 #include "nvim/state.h"
 #include "nvim/vim.h"
 #include "nvim/ascii.h"
+#include "nvim/mark.h"
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "cursor.c.generated.h"
@@ -227,9 +231,10 @@ static int coladvance2(
     }
   }
 
-  /* prevent from moving onto a trail byte */
-  if (has_mbyte)
-    mb_adjustpos(curbuf, pos);
+  // Prevent from moving onto a trail byte.
+  if (has_mbyte) {
+    mark_mb_adjustpos(curbuf, pos);
+  }
 
   if (col < wcol)
     return FAIL;
@@ -294,6 +299,26 @@ linenr_T get_cursor_rel_lnum(win_T *wp, linenr_T lnum)
   return (lnum < cursor) ? -retval : retval;
 }
 
+// Make sure "pos.lnum" and "pos.col" are valid in "buf".
+// This allows for the col to be on the NUL byte.
+void check_pos(buf_T *buf, pos_T *pos)
+{
+  char_u *line;
+  colnr_T len;
+
+  if (pos->lnum > buf->b_ml.ml_line_count) {
+     pos->lnum = buf->b_ml.ml_line_count;
+  }
+
+  if (pos->col > 0) {
+     line = ml_get_buf(buf, pos->lnum, false);
+     len = (colnr_T)STRLEN(line);
+     if (pos->col > len) {
+         pos->col = len;
+     }
+  }
+}
+
 /*
  * Make sure curwin->w_cursor.lnum is valid.
  */
@@ -318,9 +343,8 @@ void check_cursor_col(void)
   check_cursor_col_win(curwin);
 }
 
-/*
- * Make sure win->w_cursor.col is valid.
- */
+/// Make sure win->w_cursor.col is valid. Special handling of insert-mode.
+/// @see mb_check_adjust_col
 void check_cursor_col_win(win_T *win)
 {
   colnr_T len;
@@ -342,9 +366,10 @@ void check_cursor_col_win(win_T *win)
       win->w_cursor.col = len;
     } else {
       win->w_cursor.col = len - 1;
-      /* Move the cursor to the head byte. */
-      if (has_mbyte)
-        mb_adjustpos(win->w_buffer, &win->w_cursor);
+      // Move the cursor to the head byte.
+      if (has_mbyte) {
+        mark_mb_adjustpos(win->w_buffer, &win->w_cursor);
+      }
     }
   } else if (win->w_cursor.col < 0) {
     win->w_cursor.col = 0;
