@@ -1,3 +1,6 @@
+// This is an open source non-commercial project. Dear PVS-Studio, please check
+// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
+
 /// @file digraph.c
 ///
 /// code for digraphs
@@ -1448,10 +1451,8 @@ int get_digraph(int cmdline)
 {
   int cc;
   no_mapping++;
-  allow_keys++;
   int c = plain_vgetc();
   no_mapping--;
-  allow_keys--;
 
   if (c != ESC) {
     // ESC cancels CTRL-K
@@ -1468,10 +1469,8 @@ int get_digraph(int cmdline)
       add_to_showcmd(c);
     }
     no_mapping++;
-    allow_keys++;
     cc = plain_vgetc();
     no_mapping--;
-    allow_keys--;
 
     if (cc != ESC) {
       // ESC cancels CTRL-K
@@ -1573,7 +1572,8 @@ int getdigraph(int char1, int char2, int meta_char)
 
   if (((retval = getexactdigraph(char1, char2, meta_char)) == char2)
       && (char1 != char2)
-      && ((retval = getexactdigraph(char2, char1, meta_char)) == char1)) {
+      && ((retval = getexactdigraph(char2, char1, meta_char))  // -V764
+          == char1)) {
     return char2;
   }
   return retval;
@@ -1679,11 +1679,7 @@ static void printdigraph(digr_T *dp)
 
   int list_width;
 
-  if ((dy_flags & DY_UHEX) || has_mbyte) {
-    list_width = 13;
-  } else {
-    list_width = 11;
-  }
+  list_width = 13;
 
   if (dp->result != 0) {
     if (msg_col > Columns - list_width) {
@@ -1704,15 +1700,11 @@ static void printdigraph(digr_T *dp)
     *p++ = dp->char2;
     *p++ = ' ';
 
-    if (has_mbyte) {
-      // add a space to draw a composing char on
-      if (enc_utf8 && utf_iscomposing(dp->result)) {
-        *p++ = ' ';
-      }
-      p += (*mb_char2bytes)(dp->result, p);
-    } else {
-      *p++ = (char_u)dp->result;
+    // add a space to draw a composing char on
+    if (utf_iscomposing(dp->result)) {
+      *p++ = ' ';
     }
+    p += (*mb_char2bytes)(dp->result, p);
 
     if (char2cells(dp->result) == 1) {
       *p++ = ' ';
