@@ -34,7 +34,9 @@ local ADDR_ARGUMENTS        =  2
 local ADDR_LOADED_BUFFERS   =  3
 local ADDR_BUFFERS          =  4
 local ADDR_TABS             =  5
-local ADDR_QUICKFIX         =  6
+local ADDR_TABS_RELATIVE    =  6
+local ADDR_QUICKFIX         =  7
+local ADDR_OTHER            =  99
 
 -- The following table is described in ex_cmds_defs.h file.
 return {
@@ -356,6 +358,12 @@ return {
     addr_type=ADDR_LINES,
     func='ex_cbuffer',
   },
+  {
+    command='cbottom',
+    flags=bit.bor(TRLBAR),
+    addr_type=ADDR_LINES,
+    func='ex_cbottom',
+  },
   {
     command='cc',
     flags=bit.bor(RANGE, NOTADR, COUNT, TRLBAR, BANG),
@@ -454,6 +462,12 @@ return {
     addr_type=ADDR_LINES,
     func='ex_checktime',
   },
+  {
+    command='chistory',
+    flags=bit.bor(TRLBAR),
+    addr_type=ADDR_LINES,
+    func='qf_history',
+  },
   {
     command='clist',
     flags=bit.bor(BANG, EXTRA, TRLBAR, CMDWIN),
@@ -472,6 +486,12 @@ return {
     addr_type=ADDR_WINDOWS,
     func='ex_close',
   },
+  {
+    command='clearjumps',
+    flags=bit.bor(TRLBAR, CMDWIN),
+    addr_type=ADDR_LINES,
+    func='ex_clearjumps',
+  },
   {
     command='cmap',
     flags=bit.bor(EXTRA, TRLBAR, NOTRLCOM, USECTRLV, CMDWIN),
@@ -910,6 +930,12 @@ return {
     addr_type=ADDR_LINES,
     func='ex_filetype',
   },
+  {
+    command='filter',
+    flags=bit.bor(BANG, NEEDARG, EXTRA, NOTRLCOM),
+    addr_type=ADDR_LINES,
+    func='ex_wrongmodifier',
+  },
   {
     command='find',
     flags=bit.bor(RANGE, NOTADR, BANG, FILE1, EDITCMD, ARGOPT, TRLBAR),
@@ -1270,6 +1296,12 @@ return {
     addr_type=ADDR_LINES,
     func='ex_cbuffer',
   },
+  {
+    command='lbottom',
+    flags=bit.bor(TRLBAR),
+    addr_type=ADDR_LINES,
+    func='ex_cbottom',
+  },
   {
     command='lcd',
     flags=bit.bor(BANG, FILE1, TRLBAR, CMDWIN),
@@ -1380,6 +1412,12 @@ return {
     addr_type=ADDR_LINES,
     func='ex_helpgrep',
   },
+  {
+    command='lhistory',
+    flags=bit.bor(TRLBAR),
+    addr_type=ADDR_LINES,
+    func='qf_history',
+  },
   {
     command='ll',
     flags=bit.bor(RANGE, NOTADR, COUNT, TRLBAR, BANG),
@@ -1604,8 +1642,8 @@ return {
   },
   {
     command='messages',
-    flags=bit.bor(TRLBAR, CMDWIN),
-    addr_type=ADDR_LINES,
+    flags=bit.bor(EXTRA, TRLBAR, RANGE, CMDWIN),
+    addr_type=ADDR_OTHER,
     func='ex_messages',
   },
   {
@@ -2619,12 +2657,12 @@ return {
   {
     command='tab',
     flags=bit.bor(NEEDARG, EXTRA, NOTRLCOM),
-    addr_type=ADDR_LINES,
+    addr_type=ADDR_TABS,
     func='ex_wrongmodifier',
   },
   {
     command='tabclose',
-    flags=bit.bor(RANGE, NOTADR, COUNT, BANG, TRLBAR, CMDWIN),
+    flags=bit.bor(BANG, RANGE, NOTADR, ZEROR, EXTRA, NOSPC, TRLBAR, CMDWIN),
     addr_type=ADDR_TABS,
     func='ex_tabclose',
   },
@@ -2649,7 +2687,7 @@ return {
   {
     command='tabfirst',
     flags=bit.bor(TRLBAR),
-    addr_type=ADDR_LINES,
+    addr_type=ADDR_TABS,
     func='ex_tabnext',
   },
   {
@@ -2661,13 +2699,13 @@ return {
   {
     command='tablast',
     flags=bit.bor(TRLBAR),
-    addr_type=ADDR_LINES,
+    addr_type=ADDR_TABS,
     func='ex_tabnext',
   },
   {
     command='tabnext',
-    flags=bit.bor(RANGE, NOTADR, COUNT, TRLBAR),
-    addr_type=ADDR_LINES,
+    flags=bit.bor(RANGE, NOTADR, ZEROR, EXTRA, NOSPC, TRLBAR),
+    addr_type=ADDR_TABS,
     func='ex_tabnext',
   },
   {
@@ -2678,32 +2716,32 @@ return {
   },
   {
     command='tabonly',
-    flags=bit.bor(BANG, RANGE, NOTADR, TRLBAR, CMDWIN),
+    flags=bit.bor(BANG, RANGE, NOTADR, ZEROR, EXTRA, NOSPC, TRLBAR, CMDWIN),
     addr_type=ADDR_TABS,
     func='ex_tabonly',
   },
   {
     command='tabprevious',
-    flags=bit.bor(RANGE, NOTADR, COUNT, TRLBAR),
-    addr_type=ADDR_LINES,
+    flags=bit.bor(RANGE, NOTADR, ZEROR, EXTRA, NOSPC, TRLBAR),
+    addr_type=ADDR_TABS_RELATIVE,
     func='ex_tabnext',
   },
   {
     command='tabNext',
-    flags=bit.bor(RANGE, NOTADR, COUNT, TRLBAR),
-    addr_type=ADDR_LINES,
+    flags=bit.bor(RANGE, NOTADR, ZEROR, EXTRA, NOSPC, TRLBAR),
+    addr_type=ADDR_TABS_RELATIVE,
     func='ex_tabnext',
   },
   {
     command='tabrewind',
     flags=bit.bor(TRLBAR),
-    addr_type=ADDR_LINES,
+    addr_type=ADDR_TABS,
     func='ex_tabnext',
   },
   {
     command='tabs',
     flags=bit.bor(TRLBAR, CMDWIN),
-    addr_type=ADDR_LINES,
+    addr_type=ADDR_TABS,
     func='ex_tabs',
   },
   {
