@@ -1,6 +1,11 @@
-/*
- * ex_eval.c: functions for Ex command line for the +eval feature.
- */
+// This is an open source non-commercial project. Dear PVS-Studio, please check
+// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
+
+// TODO(ZyX-I): move to eval/executor
+
+/// @file ex_eval.c
+///
+/// Functions for Ex command line for the +eval feature.
 #include <assert.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -12,6 +17,7 @@
 #include "nvim/ex_eval.h"
 #include "nvim/charset.h"
 #include "nvim/eval.h"
+#include "nvim/eval/typval.h"
 #include "nvim/ex_cmds2.h"
 #include "nvim/ex_docmd.h"
 #include "nvim/message.h"
@@ -19,8 +25,6 @@
 #include "nvim/regexp.h"
 #include "nvim/strings.h"
 
-
-
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "ex_eval.c.generated.h"
 #endif
@@ -57,12 +61,14 @@
  * is an error exception.)  -  The macros can be defined as expressions checking
  * for a variable that is allowed to be changed during execution of a script.
  */
-/* Values used for the Vim release. */
-# define THROW_ON_ERROR         TRUE
-# define THROW_ON_ERROR_TRUE
-# define THROW_ON_INTERRUPT     TRUE
-# define THROW_ON_INTERRUPT_TRUE
 
+// Values used for the Vim release.
+#define THROW_ON_ERROR true
+#define THROW_ON_ERROR_TRUE
+#define THROW_ON_INTERRUPT true
+#define THROW_ON_INTERRUPT_TRUE
+
+#define discard_pending_return(p) tv_free((typval_T *)(p))
 
 /*
  * When several errors appear in a row, setting "force_abort" is delayed until
@@ -482,7 +488,7 @@ static int throw_exception(void *value, int type, char_u *cmdname)
       msg_scroll = TRUE;            /* always scroll up, don't overwrite */
 
     smsg(_("Exception thrown: %s"), excp->value);
-    msg_puts((char_u *)"\n");       /* don't overwrite this either */
+    msg_puts("\n");  // don't overwrite this either
 
     if (debug_break_level > 0 || *p_vfile == NUL)
       cmdline_row = msg_row;
@@ -532,15 +538,17 @@ static void discard_exception(except_T *excp, int was_finished)
     smsg(was_finished ? _("Exception finished: %s")
                       : _("Exception discarded: %s"),
          excp->value);
-    msg_puts((char_u *)"\n");       /* don't overwrite this either */
-    if (debug_break_level > 0 || *p_vfile == NUL)
+    msg_puts("\n");  // don't overwrite this either
+    if (debug_break_level > 0 || *p_vfile == NUL) {
       cmdline_row = msg_row;
-    --no_wait_return;
-    if (debug_break_level > 0)
+    }
+    no_wait_return--;
+    if (debug_break_level > 0) {
       msg_silent = save_msg_silent;
-    else
+    } else {
       verbose_leave();
-    STRCPY(IObuff, saved_IObuff);
+    }
+    xstrlcpy((char *)IObuff, (const char *)saved_IObuff, IOSIZE);
     xfree(saved_IObuff);
   }
   if (excp->type != ET_INTERRUPT)
@@ -595,7 +603,7 @@ static void catch_exception(except_T *excp)
       msg_scroll = TRUE;            /* always scroll up, don't overwrite */
 
     smsg(_("Exception caught: %s"), excp->value);
-    msg_puts((char_u *)"\n");       /* don't overwrite this either */
+    msg_puts("\n");  // don't overwrite this either
 
     if (debug_break_level > 0 || *p_vfile == NUL)
       cmdline_row = msg_row;
@@ -714,7 +722,7 @@ static void report_pending(int action, int pending, void *value)
   ++no_wait_return;
   msg_scroll = TRUE;            /* always scroll up, don't overwrite */
   smsg(mesg, s);
-  msg_puts((char_u *)"\n");     /* don't overwrite this either */
+  msg_puts("\n");  // don't overwrite this either
   cmdline_row = msg_row;
   --no_wait_return;
   if (debug_break_level > 0)
@@ -777,7 +785,6 @@ void report_discard_pending(int pending, void *value)
  */
 void ex_if(exarg_T *eap)
 {
-  int error;
   int skip;
   int result;
   struct condstack    *cstack = eap->cstack;
@@ -798,6 +805,7 @@ void ex_if(exarg_T *eap)
                                                                       1] &
                                                      CSF_ACTIVE));
 
+    bool error;
     result = eval_to_bool(eap->arg, &error, &eap->nextcmd, skip);
 
     if (!skip && !error) {
@@ -842,7 +850,6 @@ void ex_endif(exarg_T *eap)
  */
 void ex_else(exarg_T *eap)
 {
-  int error;
   int skip;
   int result;
   struct condstack    *cstack = eap->cstack;
@@ -899,6 +906,7 @@ void ex_else(exarg_T *eap)
   }
 
   if (eap->cmdidx == CMD_elseif) {
+    bool error;
     result = eval_to_bool(eap->arg, &error, &eap->nextcmd, skip);
     /* When throwing error exceptions, we want to throw always the first
      * of several errors in a row.  This is what actually happens when
@@ -923,7 +931,7 @@ void ex_else(exarg_T *eap)
  */
 void ex_while(exarg_T *eap)
 {
-  int error;
+  bool error;
   int skip;
   int result;
   struct condstack    *cstack = eap->cstack;
@@ -1145,23 +1153,25 @@ void ex_endwhile(exarg_T *eap)
  */
 void ex_throw(exarg_T *eap)
 {
-  char_u      *arg = eap->arg;
-  char_u      *value;
+  const char *arg = (const char *)eap->arg;
+  char *value;
 
-  if (*arg != NUL && *arg != '|' && *arg != '\n')
-    value = eval_to_string_skip(arg, &eap->nextcmd, eap->skip);
-  else {
+  if (*arg != NUL && *arg != '|' && *arg != '\n') {
+    value = eval_to_string_skip(arg, (const char **)&eap->nextcmd,
+                                (bool)eap->skip);
+  } else {
     EMSG(_(e_argreq));
     value = NULL;
   }
 
-  /* On error or when an exception is thrown during argument evaluation, do
-   * not throw. */
+  // On error or when an exception is thrown during argument evaluation, do
+  // not throw.
   if (!eap->skip && value != NULL) {
-    if (throw_exception(value, ET_USER, NULL) == FAIL)
+    if (throw_exception((char_u *)value, ET_USER, NULL) == FAIL) {
       xfree(value);
-    else
+    } else {
       do_throw(eap->cstack);
+    }
   }
 }
 
