@@ -1,3 +1,6 @@
+// This is an open source non-commercial project. Dear PVS-Studio, please check
+// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
+
 // vim: set fdm=marker fdl=1 fdc=3
 
 /*
@@ -29,6 +32,7 @@
 #include "nvim/strings.h"
 #include "nvim/syntax.h"
 #include "nvim/undo.h"
+#include "nvim/ops.h"
 
 /* local declarations. {{{1 */
 /* typedef fold_T {{{2 */
@@ -165,7 +169,7 @@ bool hasFoldingWin(
   int use_level = FALSE;
   int maybe_small = FALSE;
   garray_T    *gap;
-  int low_level = 0;;
+  int low_level = 0;
 
   checkupdate(win);
   /*
@@ -1438,13 +1442,16 @@ static void foldMarkAdjustRecurse(garray_T *gap, linenr_T line1, linenr_T line2,
         } else {
           /* 5. fold is below line1 and contains line2; need to
            * correct nested folds too */
-          foldMarkAdjustRecurse(&fp->fd_nested, line1 - fp->fd_top,
-              line2 - fp->fd_top, amount,
-              amount_after + (fp->fd_top - top));
           if (amount == MAXLNUM) {
+            foldMarkAdjustRecurse(&fp->fd_nested, line1 - fp->fd_top,
+                                  line2 - fp->fd_top, amount,
+                                  amount_after + (fp->fd_top - top));
             fp->fd_len -= line2 - fp->fd_top + 1;
             fp->fd_top = line1;
           } else {
+            foldMarkAdjustRecurse(&fp->fd_nested, line1 - fp->fd_top,
+                                  line2 - fp->fd_top, amount,
+                                  amount_after - amount);
             fp->fd_len += amount_after - amount;
             fp->fd_top += amount;
           }
@@ -1590,29 +1597,32 @@ static void foldCreateMarkers(linenr_T start, linenr_T end)
 /*
  * Add "marker[markerlen]" in 'commentstring' to line "lnum".
  */
-static void foldAddMarker(linenr_T lnum, char_u *marker, size_t markerlen)
+static void foldAddMarker(linenr_T lnum, const char_u *marker, size_t markerlen)
 {
   char_u      *cms = curbuf->b_p_cms;
   char_u      *line;
   char_u      *newline;
   char_u      *p = (char_u *)strstr((char *)curbuf->b_p_cms, "%s");
+  bool line_is_comment = false;
 
-  /* Allocate a new line: old-line + 'cms'-start + marker + 'cms'-end */
+  // Allocate a new line: old-line + 'cms'-start + marker + 'cms'-end
   line = ml_get(lnum);
   size_t line_len = STRLEN(line);
 
   if (u_save(lnum - 1, lnum + 1) == OK) {
+    // Check if the line ends with an unclosed comment
+    skip_comment(line, false, false, &line_is_comment);
     newline = xmalloc(line_len + markerlen + STRLEN(cms) + 1);
     STRCPY(newline, line);
-    if (p == NULL)
+    // Append the marker to the end of the line
+    if (p == NULL || line_is_comment) {
       STRLCPY(newline + line_len, marker, markerlen + 1);
-    else {
+    } else {
       STRCPY(newline + line_len, cms);
-      STRNCPY(newline + line_len + (p - cms), marker, markerlen);
+      memcpy(newline + line_len + (p - cms), marker, markerlen);
       STRCPY(newline + line_len + (p - cms) + markerlen, p + 2);
     }
-
-    ml_replace(lnum, newline, FALSE);
+    ml_replace(lnum, newline, false);
   }
 }
 
@@ -1673,7 +1683,8 @@ static void foldDelMarker(linenr_T lnum, char_u *marker, size_t markerlen)
     if (u_save(lnum - 1, lnum + 1) == OK) {
       /* Make new line: text-before-marker + text-after-marker */
       newline = xmalloc(STRLEN(line) - len + 1);
-      STRNCPY(newline, line, p - line);
+      assert(p >= line);
+      memcpy(newline, line, (size_t)(p - line));
       STRCPY(newline + (p - line), p + len);
       ml_replace(lnum, newline, FALSE);
     }
@@ -1681,12 +1692,10 @@ static void foldDelMarker(linenr_T lnum, char_u *marker, size_t markerlen)
   }
 }
 
-/* get_foldtext() {{{2 */
-/*
- * Return the text for a closed fold at line "lnum", with last line "lnume".
- * When 'foldtext' isn't set puts the result in "buf[51]".  Otherwise the
- * result is in allocated memory.
- */
+// get_foldtext() {{{2
+/// Return the text for a closed fold at line "lnum", with last line "lnume".
+/// When 'foldtext' isn't set puts the result in "buf[FOLD_TEXT_LEN]".
+/// Otherwise the result is in allocated memory.
 char_u *get_foldtext(win_T *wp, linenr_T lnum, linenr_T lnume,
                      foldinfo_T *foldinfo, char_u *buf)
   FUNC_ATTR_NONNULL_ARG(1)
@@ -1773,8 +1782,12 @@ char_u *get_foldtext(win_T *wp, linenr_T lnum, linenr_T lnume,
     }
   }
   if (text == NULL) {
-    sprintf((char *)buf, _("+--%3ld lines folded "),
-        (long)(lnume - lnum + 1));
+    unsigned long count = (unsigned long)(lnume - lnum + 1);
+
+    vim_snprintf((char *)buf, FOLD_TEXT_LEN,
+                 ngettext("+--%3ld line folded",
+                          "+--%3ld lines folded ", count),
+                 count);
     text = buf;
   }
   return text;
@@ -2224,32 +2237,51 @@ static linenr_T foldUpdateIEMSRecurse(garray_T *gap, int level,
              * before where we started looking, extend it.  If it
              * starts at another line, update nested folds to keep
              * their position, compensating for the new fd_top. */
-            if (fp->fd_top >= startlnum && fp->fd_top != firstlnum) {
-              if (fp->fd_top > firstlnum)
-                /* like lines are inserted */
+            if (fp->fd_top == firstlnum) {
+              // We have found a fold beginning exactly where we want one.
+            } else if (fp->fd_top >= startlnum) {
+              if (fp->fd_top > firstlnum) {
+                // We will move the start of this fold up, hence we move all
+                // nested folds (with relative line numbers) down.
                 foldMarkAdjustRecurse(&fp->fd_nested,
-                    (linenr_T)0, (linenr_T)MAXLNUM,
-                    (long)(fp->fd_top - firstlnum), 0L);
-              else
-                /* like lines are deleted */
+                                      (linenr_T)0, (linenr_T)MAXLNUM,
+                                      (long)(fp->fd_top - firstlnum), 0L);
+              } else {
+                // Will move fold down, move nested folds relatively up.
                 foldMarkAdjustRecurse(&fp->fd_nested,
-                    (linenr_T)0,
-                    (long)(firstlnum - fp->fd_top - 1),
-                    (linenr_T)MAXLNUM,
-                    (long)(fp->fd_top - firstlnum));
+                                      (linenr_T)0,
+                                      (long)(firstlnum - fp->fd_top - 1),
+                                      (linenr_T)MAXLNUM,
+                                      (long)(fp->fd_top - firstlnum));
+              }
               fp->fd_len += fp->fd_top - firstlnum;
               fp->fd_top = firstlnum;
-              fold_changed = TRUE;
-            } else if (flp->start != 0 && lvl == level
-                       && fp->fd_top != firstlnum) {
-              /* Existing fold that includes startlnum must stop
-               * if we find the start of a new fold at the same
-               * level.  Split it.  Delete contained folds at
-               * this point to split them too. */
-              foldRemove(&fp->fd_nested, flp->lnum - fp->fd_top,
-                  flp->lnum - fp->fd_top);
+              fold_changed = true;
+            } else if ((flp->start != 0 && lvl == level)
+                       || (firstlnum != startlnum)) {
+              // Before there was a fold spanning from above startlnum to below
+              // firstlnum. This fold is valid above startlnum (because we are
+              // not updating that range), but there is now a break in it.
+              // If the break is because we are now forced to start a new fold
+              // at the level "level" at line fline->lnum, then we need to
+              // split the fold at fline->lnum.
+              // If the break is because the range [startlnum, firstlnum) is
+              // now at a lower indent than "level", we need to split the fold
+              // in this range.
+              // Any splits have to be done recursively.
+              linenr_T breakstart;
+              linenr_T breakend;
+              if (firstlnum != startlnum) {
+                breakstart = startlnum;
+                breakend = firstlnum;
+              } else {
+                breakstart = flp->lnum;
+                breakend = flp->lnum;
+              }
+              foldRemove(&fp->fd_nested, breakstart - fp->fd_top,
+                         breakend - fp->fd_top);
               i = (int)(fp - (fold_T *)gap->ga_data);
-              foldSplit(gap, i, flp->lnum, flp->lnum - 1);
+              foldSplit(gap, i, breakstart, breakend - 1);
               fp = (fold_T *)gap->ga_data + i + 1;
               /* If using the "marker" or "syntax" method, we
                * need to continue until the end of the fold is
@@ -2259,6 +2291,16 @@ static linenr_T foldUpdateIEMSRecurse(garray_T *gap, int level,
                   || getlevel == foldlevelSyntax)
                 finish = TRUE;
             }
+            if (fp->fd_top == startlnum && concat) {
+              i = (int)(fp - (fold_T *)gap->ga_data);
+              if (i != 0) {
+                fp2 = fp - 1;
+                if (fp2->fd_top + fp2->fd_len == fp->fd_top) {
+                  foldMerge(fp2, gap, fp);
+                  fp = fp2;
+                }
+              }
+            }
             break;
           }
           if (fp->fd_top >= startlnum) {
@@ -2499,6 +2541,8 @@ static void foldSplit(garray_T *gap, int i, linenr_T top, linenr_T bot)
 
   fp = (fold_T *)gap->ga_data + i;
   fp[1].fd_top = bot + 1;
+  // check for wrap around (MAXLNUM, and 32bit)
+  assert(fp[1].fd_top > bot);
   fp[1].fd_len = fp->fd_len - (fp[1].fd_top - fp->fd_top);
   fp[1].fd_flags = fp->fd_flags;
   fp[1].fd_small = MAYBE;
@@ -2531,10 +2575,10 @@ static void foldSplit(garray_T *gap, int i, linenr_T top, linenr_T bot)
  *      1  2  3
  *      1  2  3
  * top     2  3  4  5
- *	   2  3  4  5
- * bot	   2  3  4  5
- *	      3     5  6
- *	      3     5  6
+ *     2  3  4  5
+ * bot     2  3  4  5
+ *        3     5  6
+ *        3     5  6
  *
  * 1: not changed
  * 2: truncate to stop above "top"
@@ -2547,34 +2591,35 @@ static void foldRemove(garray_T *gap, linenr_T top, linenr_T bot)
 {
   fold_T      *fp = NULL;
 
-  if (bot < top)
-    return;             /* nothing to do */
+  if (bot < top) {
+    return;             // nothing to do
+  }
 
   for (;; ) {
-    /* Find fold that includes top or a following one. */
+    // Find fold that includes top or a following one.
     if (foldFind(gap, top, &fp) && fp->fd_top < top) {
-      /* 2: or 3: need to delete nested folds */
+      // 2: or 3: need to delete nested folds
       foldRemove(&fp->fd_nested, top - fp->fd_top, bot - fp->fd_top);
-      if (fp->fd_top + fp->fd_len > bot + 1) {
-        /* 3: need to split it. */
+      if (fp->fd_top + fp->fd_len - 1 > bot) {
+        // 3: need to split it.
         foldSplit(gap, (int)(fp - (fold_T *)gap->ga_data), top, bot);
       } else {
-        /* 2: truncate fold at "top". */
+        // 2: truncate fold at "top".
         fp->fd_len = top - fp->fd_top;
       }
-      fold_changed = TRUE;
+      fold_changed = true;
       continue;
     }
     if (fp >= (fold_T *)(gap->ga_data) + gap->ga_len
         || fp->fd_top > bot) {
-      /* 6: Found a fold below bot, can stop looking. */
+      // 6: Found a fold below bot, can stop looking.
       break;
     }
     if (fp->fd_top >= top) {
-      /* Found an entry below top. */
-      fold_changed = TRUE;
+      // Found an entry below top.
+      fold_changed = true;
       if (fp->fd_top + fp->fd_len - 1 > bot) {
-        /* 5: Make fold that includes bot start below bot. */
+        // 5: Make fold that includes bot start below bot.
         foldMarkAdjustRecurse(&fp->fd_nested,
             (linenr_T)0, (long)(bot - fp->fd_top),
             (linenr_T)MAXLNUM, (long)(fp->fd_top - bot - 1));
@@ -2583,12 +2628,163 @@ static void foldRemove(garray_T *gap, linenr_T top, linenr_T bot)
         break;
       }
 
-      /* 4: Delete completely contained fold. */
-      deleteFoldEntry(gap, (int)(fp - (fold_T *)gap->ga_data), TRUE);
+      // 4: Delete completely contained fold.
+      deleteFoldEntry(gap, (int)(fp - (fold_T *)gap->ga_data), true);
     }
   }
 }
 
+// foldMoveRange() {{{2
+static void reverse_fold_order(garray_T *gap, size_t start, size_t end)
+{
+  for (; start < end; start++, end--) {
+    fold_T *left = (fold_T *)gap->ga_data + start;
+    fold_T *right = (fold_T *)gap->ga_data + end;
+    fold_T tmp = *left;
+    *left = *right;
+    *right = tmp;
+  }
+}
+
+// Move folds within the inclusive range "line1" to "line2" to after "dest"
+// require "line1" <= "line2" <= "dest"
+//
+// There are the following situations for the first fold at or below line1 - 1.
+//       1  2  3  4
+//       1  2  3  4
+// line1    2  3  4
+//          2  3  4  5  6  7
+// line2       3  4  5  6  7
+//             3  4     6  7  8  9
+// dest           4        7  8  9
+//                4        7  8    10
+//                4        7  8    10
+//
+// In the following descriptions, "moved" means moving in the buffer, *and* in
+// the fold array.
+// Meanwhile, "shifted" just means moving in the buffer.
+// 1. not changed
+// 2. truncated above line1
+// 3. length reduced by  line2 - line1, folds starting between the end of 3 and
+//    dest are truncated and shifted up
+// 4. internal folds moved (from [line1, line2] to dest)
+// 5. moved to dest.
+// 6. truncated below line2 and moved.
+// 7. length reduced by line2 - dest, folds starting between line2 and dest are
+//    removed, top is moved down by move_len.
+// 8. truncated below dest and shifted up.
+// 9. shifted up
+// 10. not changed
+static void truncate_fold(fold_T *fp, linenr_T end)
+{
+  // I want to stop *at here*, foldRemove() stops *above* top
+  end += 1;
+  foldRemove(&fp->fd_nested, end - fp->fd_top, MAXLNUM);
+  fp->fd_len = end - fp->fd_top;
+}
+
+#define FOLD_END(fp) ((fp)->fd_top + (fp)->fd_len - 1)
+#define VALID_FOLD(fp, gap) ((fp) < ((fold_T *)(gap)->ga_data + (gap)->ga_len))
+#define FOLD_INDEX(fp, gap) ((size_t)(fp - ((fold_T *)(gap)->ga_data)))
+void foldMoveRange(garray_T *gap, const linenr_T line1, const linenr_T line2,
+                   const linenr_T dest)
+{
+  fold_T *fp;
+  const linenr_T range_len = line2 - line1 + 1;
+  const linenr_T move_len = dest - line2;
+  const bool at_start = foldFind(gap, line1 - 1, &fp);
+
+  if (at_start) {
+    if (FOLD_END(fp) > dest) {
+      // Case 4 -- don't have to change this fold, but have to move nested
+      // folds.
+      foldMoveRange(&fp->fd_nested, line1 - fp->fd_top, line2 -
+                    fp->fd_top, dest - fp->fd_top);
+      return;
+    } else if (FOLD_END(fp) > line2) {
+      // Case 3 -- Remove nested folds between line1 and line2 & reduce the
+      // length of fold by "range_len".
+      // Folds after this one must be dealt with.
+      foldMarkAdjustRecurse(&fp->fd_nested, line1 - fp->fd_top,
+                            line2 - fp->fd_top, MAXLNUM, -range_len);
+      fp->fd_len -= range_len;
+    } else {
+      // Case 2 -- truncate fold *above* line1.
+      // Folds after this one must be dealt with.
+      truncate_fold(fp, line1 - 1);
+    }
+    // Look at the next fold, and treat that one as if it were the first after
+    // "line1" (because now it is).
+    fp = fp + 1;
+  }
+
+  if (!VALID_FOLD(fp, gap) || fp->fd_top > dest) {
+    // No folds after "line1" and before "dest"
+    // Case 10.
+    return;
+  } else if (fp->fd_top > line2) {
+    for (; VALID_FOLD(fp, gap) && FOLD_END(fp) <= dest; fp++) {
+      // Case 9. (for all case 9's) -- shift up.
+      fp->fd_top -= range_len;
+    }
+    if (VALID_FOLD(fp, gap) && fp->fd_top <= dest) {
+      // Case 8. -- ensure truncated at dest, shift up
+      truncate_fold(fp, dest);
+      fp->fd_top -= range_len;
+    }
+    return;
+  } else if (FOLD_END(fp) > dest) {
+    // Case 7 -- remove nested folds and shrink
+    foldMarkAdjustRecurse(&fp->fd_nested, line2 + 1 - fp->fd_top,
+                          dest - fp->fd_top, MAXLNUM, -move_len);
+    fp->fd_len -= move_len;
+    fp->fd_top += move_len;
+    return;
+  }
+
+  // Case 5 or 6: changes rely on whether there are folds between the end of
+  // this fold and "dest".
+  size_t move_start = FOLD_INDEX(fp, gap);
+  size_t move_end = 0, dest_index = 0;
+  for (; VALID_FOLD(fp, gap) && fp->fd_top <= dest; fp++) {
+    if (fp->fd_top <= line2) {
+      // 5, or 6
+      if (FOLD_END(fp) > line2) {
+        // 6, truncate before moving
+        truncate_fold(fp, line2);
+      }
+      fp->fd_top += move_len;
+      continue;
+    }
+
+    // Record index of the first fold after the moved range.
+    if (move_end == 0) {
+      move_end = FOLD_INDEX(fp, gap);
+    }
+
+    if (FOLD_END(fp) > dest) {
+      truncate_fold(fp, dest);
+    }
+
+    fp->fd_top -= range_len;
+  }
+  dest_index = FOLD_INDEX(fp, gap);
+
+  // All folds are now correct, but not necessarily in the correct order.
+  // We must swap folds in the range [move_end, dest_index) with those in the
+  // range [move_start, move_end).
+  if (move_end == 0) {
+    // There are no folds after those moved, so none were moved out of order.
+    return;
+  }
+  reverse_fold_order(gap, move_start, dest_index - 1);
+  reverse_fold_order(gap, move_start, move_start + dest_index - move_end - 1);
+  reverse_fold_order(gap, move_start + dest_index - move_end, dest_index - 1);
+}
+#undef FOLD_END
+#undef VALID_FOLD
+#undef FOLD_INDEX
+
 /* foldMerge() {{{2 */
 /*
  * Merge two adjacent folds (and the nested ones in them).
