@@ -1,6 +1,10 @@
 #ifndef NVIM_GETCHAR_H
 #define NVIM_GETCHAR_H
 
+#include "nvim/types.h"
+#include "nvim/buffer_defs.h"
+#include "nvim/ex_cmds_defs.h"
+
 /* Values for "noremap" argument of ins_typebuf().  Also used for
  * map->m_noremap and menu->noremap[]. */
 #define REMAP_YES       0       /* allow remapping */
@@ -12,6 +16,7 @@
 #define KEYLEN_PART_MAP -2      /* keylen value for incomplete mapping */
 #define KEYLEN_REMOVED  9999    /* keylen value for removed sequence */
 
+
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "getchar.h.generated.h"
 #endif
