@@ -1,3 +1,6 @@
+// This is an open source non-commercial project. Dear PVS-Studio, please check
+// it. PVS-Studio Static Code Analyzer for C, C++ and C#: http://www.viva64.com
+
 /// An abstraction to handle blocks of memory which can be stored in a file.
 /// This is the implementation of a sort of virtual memory.
 ///
@@ -909,12 +912,7 @@ static bool mf_do_open(memfile_T *mfp, char_u *fname, int flags)
     return false;
   }
 
-#ifdef HAVE_FD_CLOEXEC
-  int fdflags = fcntl(mfp->mf_fd, F_GETFD);
-  if (fdflags >= 0 && (fdflags & FD_CLOEXEC) == 0) {
-    (void)fcntl(mfp->mf_fd, F_SETFD, fdflags | FD_CLOEXEC);
-  }
-#endif
+  (void)os_set_cloexec(mfp->mf_fd);
 #ifdef HAVE_SELINUX
   mch_copy_sec(fname, mfp->mf_fname);
 #endif
