@@ -1,26 +1,37 @@
 local helpers = require('test.functional.helpers')(after_each)
-local clear, eq, eval, execute, feed, insert, neq, next_msg, nvim,
+local clear, eq, eval, exc_exec, feed_command, feed, insert, neq, next_msg, nvim,
   nvim_dir, ok, source, write_file, mkdir, rmdir = helpers.clear,
-  helpers.eq, helpers.eval, helpers.execute, helpers.feed,
+  helpers.eq, helpers.eval, helpers.exc_exec, helpers.feed_command, helpers.feed,
   helpers.insert, helpers.neq, helpers.next_message, helpers.nvim,
   helpers.nvim_dir, helpers.ok, helpers.source,
   helpers.write_file, helpers.mkdir, helpers.rmdir
 local command = helpers.command
+local wait = helpers.wait
+local iswin = helpers.iswin
+local get_pathsep = helpers.get_pathsep
 local Screen = require('test.functional.ui.screen')
 
-if helpers.pending_win32(pending) then return end
-
 describe('jobs', function()
   local channel
 
   before_each(function()
     clear()
+    if iswin() then
+      helpers.set_shell_powershell()
+    end
     channel = nvim('get_api_info')[1]
     nvim('set_var', 'channel', channel)
     source([[
-    function! s:OnEvent(id, data, event)
+    function! Normalize(data) abort
+      " Windows: remove ^M
+      return type([]) == type(a:data)
+        \ ? map(a:data, 'substitute(v:val, "\r", "", "g")')
+        \ : a:data
+    endfunction
+    function! s:OnEvent(id, data, event) dict
       let userdata = get(self, 'user')
-      call rpcnotify(g:channel, a:event, userdata, a:data)
+      let data     = Normalize(a:data)
+      call rpcnotify(g:channel, a:event, userdata, data)
     endfunction
     let g:job_opts = {
     \ 'on_stdout': function('s:OnEvent'),
@@ -33,23 +44,36 @@ describe('jobs', function()
 
   it('uses &shell and &shellcmdflag if passed a string', function()
     nvim('command', "let $VAR = 'abc'")
-    nvim('command', "let j = jobstart('echo $VAR', g:job_opts)")
+    if iswin() then
+      nvim('command', "let j = jobstart('echo $env:VAR', g:job_opts)")
+    else
+      nvim('command', "let j = jobstart('echo $VAR', g:job_opts)")
+    end
     eq({'notification', 'stdout', {0, {'abc', ''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
   it('changes to given / directory', function()
     nvim('command', "let g:job_opts.cwd = '/'")
-    nvim('command', "let j = jobstart('pwd', g:job_opts)")
-    eq({'notification', 'stdout', {0, {'/', ''}}}, next_msg())
+    if iswin() then
+      nvim('command', "let j = jobstart('pwd|%{$_.Path}', g:job_opts)")
+    else
+      nvim('command', "let j = jobstart('pwd', g:job_opts)")
+    end
+    eq({'notification', 'stdout',
+      {0, {(iswin() and [[C:\]] or '/'), ''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
   it('changes to given `cwd` directory', function()
-    local dir = eval('resolve(tempname())')
+    local dir = eval("resolve(tempname())"):gsub("/", get_pathsep())
     mkdir(dir)
     nvim('command', "let g:job_opts.cwd = '" .. dir .. "'")
-    nvim('command', "let j = jobstart('pwd', g:job_opts)")
+    if iswin() then
+      nvim('command', "let j = jobstart('pwd|%{$_.Path}', g:job_opts)")
+    else
+      nvim('command', "let j = jobstart('pwd', g:job_opts)")
+    end
     eq({'notification', 'stdout', {0, {dir, ''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
     rmdir(dir)
@@ -59,24 +83,47 @@ describe('jobs', function()
     local dir = eval('resolve(tempname())."-bogus"')
     local _, err = pcall(function()
       nvim('command', "let g:job_opts.cwd = '" .. dir .. "'")
-      nvim('command', "let j = jobstart('pwd', g:job_opts)")
+      if iswin() then
+        nvim('command', "let j = jobstart('pwd|%{$_.Path}', g:job_opts)")
+      else
+        nvim('command', "let j = jobstart('pwd', g:job_opts)")
+      end
     end)
     ok(string.find(err, "E475: Invalid argument: expected valid directory$") ~= nil)
   end)
 
   it('returns 0 when it fails to start', function()
-    local status, rv = pcall(eval, "jobstart([])")
-    eq(false, status)
-    ok(rv ~= nil)
+    eq("", eval("v:errmsg"))
+    feed_command("let g:test_jobid = jobstart([])")
+    eq(0, eval("g:test_jobid"))
+    eq("E474:", string.match(eval("v:errmsg"), "E%d*:"))
+  end)
+
+  it('returns -1 when target is not executable #5465', function()
+    if helpers.pending_win32(pending) then return end
+    local function new_job()
+      return eval([[jobstart('')]])
+    end
+    local executable_jobid = new_job()
+    local nonexecutable_jobid = eval(
+      "jobstart(['./test/functional/fixtures/non_executable.txt'])")
+    eq(-1, nonexecutable_jobid)
+    -- Should _not_ throw an error.
+    eq("", eval("v:errmsg"))
+    -- Non-executable job should not increment the job ids. #5465
+    eq(executable_jobid + 1, new_job())
   end)
 
   it('invokes callbacks when the job writes and exits', function()
-    nvim('command', "call jobstart(['echo'], g:job_opts)")
+    -- TODO: hangs on Windows
+    if helpers.pending_win32(pending) then return end
+    nvim('command', "call jobstart('echo', g:job_opts)")
     eq({'notification', 'stdout', {0, {'', ''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
   it('allows interactive commands', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
     neq(0, eval('j'))
     nvim('command', 'call jobsend(j, "abc\\n")')
@@ -90,6 +137,7 @@ describe('jobs', function()
   end)
 
   it('preserves NULs', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     -- Make a file with NULs in it.
     local filename = helpers.tmpname()
     write_file(filename, "abc\0def\n")
@@ -107,6 +155,7 @@ describe('jobs', function()
   end)
 
   it("will not buffer data if it doesn't end in newlines", function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     if os.getenv("TRAVIS") and os.getenv("CC") == "gcc-4.9"
       and helpers.os_name() == "osx" then
       -- XXX: Hangs Travis macOS since e9061117a5b8f195c3f26a5cb94e18ddd7752d86.
@@ -122,6 +171,7 @@ describe('jobs', function()
   end)
 
   it('preserves newlines', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
     nvim('command', 'call jobsend(j, "a\\n\\nc\\n\\n\\n\\nb\\n\\n")')
     eq({'notification', 'stdout',
@@ -129,6 +179,7 @@ describe('jobs', function()
   end)
 
   it('preserves NULs', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
     nvim('command', 'call jobsend(j, ["\n123\n", "abc\\nxyz\n", ""])')
     eq({'notification', 'stdout', {0, {'\n123\n', 'abc\nxyz\n', ''}}},
@@ -138,6 +189,7 @@ describe('jobs', function()
   end)
 
   it('avoids sending final newline', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
     nvim('command', 'call jobsend(j, ["some data", "without\nfinal nl"])')
     eq({'notification', 'stdout', {0, {'some data', 'without\nfinal nl'}}},
@@ -147,12 +199,14 @@ describe('jobs', function()
   end)
 
   it('closes the job streams with jobclose', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
     nvim('command', 'call jobclose(j, "stdin")')
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
   it("disallows jobsend on a job that closed stdin", function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
     nvim('command', 'call jobclose(j, "stdin")')
     eq(false, pcall(function()
@@ -166,17 +220,20 @@ describe('jobs', function()
   end)
 
   it('disallows jobstop twice on the same job', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
     neq(0, eval('j'))
     eq(true, pcall(eval, "jobstop(j)"))
     eq(false, pcall(eval, "jobstop(j)"))
   end)
 
-  it('will not cause a memory leak if we leave a job running', function()
+  it('will not leak memory if we leave a job running', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "call jobstart(['cat', '-'], g:job_opts)")
   end)
 
   it('can get the pid value using getpid', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "let j =  jobstart(['cat', '-'], g:job_opts)")
     local pid = eval('jobpid(j)')
     eq(0,os.execute('ps -p '..pid..' > /dev/null'))
@@ -185,19 +242,21 @@ describe('jobs', function()
     neq(0,os.execute('ps -p '..pid..' > /dev/null'))
   end)
 
-  it("doesn't survive the exit of nvim", function()
+  it("do not survive the exit of nvim", function()
+    if helpers.pending_win32(pending) then return end
     -- use sleep, which doesn't die on stdin close
-    nvim('command', "let j =  jobstart(['sleep', '1000'], g:job_opts)")
-    local pid = eval('jobpid(j)')
+    nvim('command', "let g:j =  jobstart(['sleep', '1000'], g:job_opts)")
+    local pid = eval('jobpid(g:j)')
     eq(0,os.execute('ps -p '..pid..' > /dev/null'))
     clear()
     neq(0,os.execute('ps -p '..pid..' > /dev/null'))
   end)
 
   it('can survive the exit of nvim with "detach"', function()
+    if helpers.pending_win32(pending) then return end
     nvim('command', 'let g:job_opts.detach = 1')
-    nvim('command', "let j = jobstart(['sleep', '1000'], g:job_opts)")
-    local pid = eval('jobpid(j)')
+    nvim('command', "let g:j = jobstart(['sleep', '1000'], g:job_opts)")
+    local pid = eval('jobpid(g:j)')
     eq(0,os.execute('ps -p '..pid..' > /dev/null'))
     clear()
     eq(0,os.execute('ps -p '..pid..' > /dev/null'))
@@ -207,13 +266,14 @@ describe('jobs', function()
 
   it('can pass user data to the callback', function()
     nvim('command', 'let g:job_opts.user = {"n": 5, "s": "str", "l": [1]}')
-    nvim('command', "call jobstart(['echo'], g:job_opts)")
+    nvim('command', [[call jobstart('echo "foo"', g:job_opts)]])
     local data = {n = 5, s = 'str', l = {1}}
-    eq({'notification', 'stdout', {data, {'', ''}}}, next_msg())
+    eq({'notification', 'stdout', {data, {'foo', ''}}}, next_msg())
     eq({'notification', 'exit', {data, 0}}, next_msg())
   end)
 
   it('can omit options', function()
+    if helpers.pending_win32(pending) then return end
     neq(0, nvim('eval', 'delete(".Xtestjob")'))
     nvim('command', "call jobstart(['touch', '.Xtestjob'])")
     nvim('command', "sleep 100m")
@@ -224,20 +284,20 @@ describe('jobs', function()
     nvim('command', 'unlet g:job_opts.on_stdout')
     nvim('command', 'unlet g:job_opts.on_stderr')
     nvim('command', 'let g:job_opts.user = 5')
-    nvim('command', "call jobstart(['echo'], g:job_opts)")
+    nvim('command', [[call jobstart('echo "foo"', g:job_opts)]])
     eq({'notification', 'exit', {5, 0}}, next_msg())
   end)
 
   it('can omit exit callback', function()
     nvim('command', 'unlet g:job_opts.on_exit')
     nvim('command', 'let g:job_opts.user = 5')
-    nvim('command', "call jobstart(['echo'], g:job_opts)")
-    eq({'notification', 'stdout', {5, {'', ''}}}, next_msg())
+    nvim('command', [[call jobstart('echo "foo"', g:job_opts)]])
+    eq({'notification', 'stdout', {5, {'foo', ''}}}, next_msg())
   end)
 
   it('will pass return code with the exit event', function()
     nvim('command', 'let g:job_opts.user = 5')
-    nvim('command', "call jobstart([&sh, '-c', 'exit 55'], g:job_opts)")
+    nvim('command', "call jobstart('exit 55', g:job_opts)")
     eq({'notification', 'exit', {5, 55}}, next_msg())
   end)
 
@@ -247,15 +307,19 @@ describe('jobs', function()
     function g:dict.on_exit(id, code, event)
       call rpcnotify(g:channel, a:event, a:code, self.id)
     endfunction
-    call jobstart([&sh, '-c', 'exit 45'], g:dict)
+    call jobstart('exit 45', g:dict)
     ]])
     eq({'notification', 'exit', {45, 10}}, next_msg())
   end)
 
-  it('cannot redefine callbacks being used by a job', function()
+  it('can redefine callbacks being used by a job', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     local screen = Screen.new()
     screen:attach()
-    local script = [[
+    screen:set_default_attr_ids({
+      [1] = {bold=true, foreground=Screen.colors.Blue},
+    })
+    source([[
       function! g:JobHandler(job_id, data, event)
       endfunction
 
@@ -265,36 +329,98 @@ describe('jobs', function()
       \ 'on_exit': function('g:JobHandler')
       \ }
       let job = jobstart('cat -', g:callbacks)
-    ]]
-    source(script)
-    feed(':function! g:JobHandler(job_id, data, event)<cr>')
-    feed(':endfunction<cr>')
-    screen:expect([[
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      ~                                                    |
-      :function! g:JobHandler(job_id, data, event)         |
-      :  :endfunction                                      |
-      E127: Cannot redefine function JobHandler: It is in u|
-      se                                                   |
-      Press ENTER or type command to continue^              |
     ]])
+    wait()
+    source([[
+      function! g:JobHandler(job_id, data, event)
+      endfunction
+    ]])
+
+    eq("", eval("v:errmsg"))
+  end)
+
+  it('requires funcrefs for script-local (s:) functions', function()
+    -- Pass job callback names _without_ `function(...)`.
+    source([[
+      function! s:OnEvent(id, data, event) dict
+        let g:job_result = get(self, 'user')
+      endfunction
+      let s:job = jobstart('echo "foo"', {
+        \ 'on_stdout': 's:OnEvent',
+        \ 'on_stderr': 's:OnEvent',
+        \ 'on_exit':   's:OnEvent',
+        \ 'user': 2349
+        \ })
+    ]])
+
+    -- The behavior is asynchronous, retry until a time limit.
+    helpers.retry(nil, 10000, function()
+      eq("E120:", string.match(eval("v:errmsg"), "E%d*:"))
+    end)
+  end)
+
+  it('does not repeat output with slow output handlers', function()
+    if helpers.pending_win32(pending) then return end
+    source([[
+      let d = {'data': []}
+      function! d.on_stdout(job, data, event) dict
+        call add(self.data, a:data)
+        sleep 200m
+      endfunction
+      if has('win32')
+        let cmd = '1,2,3,4,5 | foreach-object -process {echo $_; sleep 0.1}'
+      else
+        let cmd = ['sh', '-c', 'for i in $(seq 1 5); do echo $i; sleep 0.1; done']
+      endif
+      call jobwait([jobstart(cmd, d)])
+      call rpcnotify(g:channel, 'data', d.data)
+    ]])
+    eq({'notification', 'data', {{{'1', ''}, {'2', ''}, {'3', ''}, {'4', ''}, {'5', ''}}}}, next_msg())
+  end)
+
+  it('jobstart() works with partial functions', function()
+    source([[
+    function PrintArgs(a1, a2, id, data, event)
+      " Windows: remove ^M
+      let normalized = map(a:data, 'substitute(v:val, "\r", "", "g")')
+      call rpcnotify(g:channel, '1', a:a1,  a:a2, normalized, a:event)
+    endfunction
+    let Callback = function('PrintArgs', ["foo", "bar"])
+    let g:job_opts = {'on_stdout': Callback}
+    call jobstart('echo "some text"', g:job_opts)
+    ]])
+    eq({'notification', '1', {'foo', 'bar', {'some text', ''}, 'stdout'}}, next_msg())
+  end)
+
+  it('jobstart() works with closures', function()
+    source([[
+      fun! MkFun()
+          let a1 = 'foo'
+          let a2 = 'bar'
+          return {id, data, event -> rpcnotify(g:channel, '1', a1, a2, Normalize(data), event)}
+      endfun
+      let g:job_opts = {'on_stdout': MkFun()}
+      call jobstart('echo "some text"', g:job_opts)
+    ]])
+    eq({'notification', '1', {'foo', 'bar', {'some text', ''}, 'stdout'}}, next_msg())
+  end)
+
+  it('jobstart() works when closure passed directly to `jobstart`', function()
+    source([[
+      let g:job_opts = {'on_stdout': {id, data, event -> rpcnotify(g:channel, '1', 'foo', 'bar', Normalize(data), event)}}
+      call jobstart('echo "some text"', g:job_opts)
+    ]])
+    eq({'notification', '1', {'foo', 'bar', {'some text', ''}, 'stdout'}}, next_msg())
   end)
 
   describe('jobwait', function()
     it('returns a list of status codes', function()
       source([[
       call rpcnotify(g:channel, 'wait', jobwait([
-      \  jobstart([&sh, '-c', 'sleep 0.10; exit 4']),
-      \  jobstart([&sh, '-c', 'sleep 0.110; exit 5']),
-      \  jobstart([&sh, '-c', 'sleep 0.210; exit 6']),
-      \  jobstart([&sh, '-c', 'sleep 0.310; exit 7'])
+      \  jobstart('sleep 0.10; exit 4'),
+      \  jobstart('sleep 0.110; exit 5'),
+      \  jobstart('sleep 0.210; exit 6'),
+      \  jobstart('sleep 0.310; exit 7')
       \  ]))
       ]])
       eq({'notification', 'wait', {{4, 5, 6, 7}}}, next_msg())
@@ -304,17 +430,17 @@ describe('jobs', function()
       source([[
       let g:dict = {'id': 10}
       let g:exits = 0
-      function g:dict.on_exit(id, code)
+      function g:dict.on_exit(id, code, event)
         if a:code != 5
           throw 'Error!'
         endif
         let g:exits += 1
       endfunction
       call jobwait([
-      \  jobstart([&sh, '-c', 'sleep 0.010; exit 5'], g:dict),
-      \  jobstart([&sh, '-c', 'sleep 0.030; exit 5'], g:dict),
-      \  jobstart([&sh, '-c', 'sleep 0.050; exit 5'], g:dict),
-      \  jobstart([&sh, '-c', 'sleep 0.070; exit 5'], g:dict)
+      \  jobstart('sleep 0.010; exit 5', g:dict),
+      \  jobstart('sleep 0.030; exit 5', g:dict),
+      \  jobstart('sleep 0.050; exit 5', g:dict),
+      \  jobstart('sleep 0.070; exit 5', g:dict)
       \  ])
       call rpcnotify(g:channel, 'wait', g:exits)
       ]])
@@ -324,10 +450,10 @@ describe('jobs', function()
     it('will return status codes in the order of passed ids', function()
       source([[
       call rpcnotify(g:channel, 'wait', jobwait([
-      \  jobstart([&sh, '-c', 'sleep 0.070; exit 4']),
-      \  jobstart([&sh, '-c', 'sleep 0.050; exit 5']),
-      \  jobstart([&sh, '-c', 'sleep 0.030; exit 6']),
-      \  jobstart([&sh, '-c', 'sleep 0.010; exit 7'])
+      \  jobstart('sleep 0.070; exit 4'),
+      \  jobstart('sleep 0.050; exit 5'),
+      \  jobstart('sleep 0.030; exit 6'),
+      \  jobstart('sleep 0.010; exit 7')
       \  ]))
       ]])
       eq({'notification', 'wait', {{4, 5, 6, 7}}}, next_msg())
@@ -337,26 +463,27 @@ describe('jobs', function()
       source([[
       call rpcnotify(g:channel, 'wait', jobwait([
       \  -10,
-      \  jobstart([&sh, '-c', 'sleep 0.01; exit 5']),
+      \  jobstart('sleep 0.01; exit 5'),
       \  ]))
       ]])
       eq({'notification', 'wait', {{-3, 5}}}, next_msg())
     end)
 
     it('will return -2 when interrupted', function()
-      execute('call rpcnotify(g:channel, "ready") | '..
+      feed_command('call rpcnotify(g:channel, "ready") | '..
               'call rpcnotify(g:channel, "wait", '..
-              'jobwait([jobstart([&sh, "-c", "sleep 10; exit 55"])]))')
+              'jobwait([jobstart("sleep 10; exit 55")]))')
       eq({'notification', 'ready', {}}, next_msg())
       feed('<c-c>')
       eq({'notification', 'wait', {{-2}}}, next_msg())
     end)
 
     it('can be called recursively', function()
+      if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
       source([[
       let g:opts = {}
       let g:counter = 0
-      function g:opts.on_stdout(id, msg)
+      function g:opts.on_stdout(id, msg, _event)
         if self.state == 0
           if self.counter < 10
             call Run()
@@ -374,7 +501,7 @@ describe('jobs', function()
           call jobclose(a:id, 'stdin')
         endif
       endfunction
-      function g:opts.on_exit()
+      function g:opts.on_exit(...)
         call rpcnotify(g:channel, 'w', printf('job %d exited', self.counter))
       endfunction
       function Run()
@@ -383,11 +510,11 @@ describe('jobs', function()
         let j.state = 0
         let j.counter = g:counter
         call jobwait([
-        \   jobstart([&sh, '-c', 'echo ready; cat -'], j),
+        \   jobstart('echo ready; cat -', j),
         \ ])
       endfunction
       ]])
-      execute('call Run()')
+      feed_command('call Run()')
       local r
       for i = 10, 1, -1 do
         r = next_msg()
@@ -399,11 +526,12 @@ describe('jobs', function()
     end)
 
     describe('with timeout argument', function()
+      if helpers.pending_win32(pending) then return end
       it('will return -1 if the wait timed out', function()
         source([[
         call rpcnotify(g:channel, 'wait', jobwait([
-        \  jobstart([&sh, '-c', 'exit 4']),
-        \  jobstart([&sh, '-c', 'sleep 10; exit 5']),
+        \  jobstart('exit 4'),
+        \  jobstart('sleep 10; exit 5'),
         \  ], 100))
         ]])
         eq({'notification', 'wait', {{4, -1}}}, next_msg())
@@ -412,8 +540,8 @@ describe('jobs', function()
       it('can pass 0 to check if a job exists', function()
         source([[
         call rpcnotify(g:channel, 'wait', jobwait([
-        \  jobstart([&sh, '-c', 'sleep 0.05; exit 4']),
-        \  jobstart([&sh, '-c', 'sleep 0.3; exit 5']),
+        \  jobstart('sleep 0.05; exit 4'),
+        \  jobstart('sleep 0.3; exit 5'),
         \  ], 0))
         ]])
         eq({'notification', 'wait', {{-1, -1}}}, next_msg())
@@ -432,6 +560,7 @@ describe('jobs', function()
   end)
 
   it('cannot have both rpc and pty options', function()
+    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     command("let g:job_opts.pty = v:true")
     command("let g:job_opts.rpc = v:true")
     local _, err = pcall(command, "let j = jobstart(['cat', '-'], g:job_opts)")
@@ -439,6 +568,7 @@ describe('jobs', function()
   end)
 
   describe('running tty-test program', function()
+    if helpers.pending_win32(pending) then return end
     local function next_chunk()
       local rv
       while true do
@@ -461,8 +591,14 @@ describe('jobs', function()
     end
 
     before_each(function()
-      -- the full path to tty-test seems to be required when running on travis.
-      insert(nvim_dir .. '/tty-test')
+      -- Redefine Normalize() so that TTY data is not munged.
+      source([[
+      function! Normalize(data) abort
+        return a:data
+      endfunction
+      ]])
+      local ext = iswin() and '.exe' or ''
+      insert(nvim_dir..'/tty-test'..ext)  -- Full path to tty-test.
       nvim('command', 'let g:job_opts.pty = 1')
       nvim('command', 'let exec = [expand("<cfile>:p")]')
       nvim('command', "let j = jobstart(exec, g:job_opts)")
@@ -481,10 +617,31 @@ describe('jobs', function()
       eq('rows: 40, cols: 10', next_chunk())
     end)
 
-    it('calling jobclose()', function()
-      -- this should send SIGHUP to the process
+    it('jobclose() sends SIGHUP', function()
       nvim('command', 'call jobclose(j)')
-      eq({'notification', 'exit', {0, 1}}, next_msg())
+      local msg = next_msg()
+      msg = (msg[2] == 'stdout') and next_msg() or msg  -- Skip stdout, if any.
+      eq({'notification', 'exit', {0, 42}}, msg)
+    end)
+
+    it('jobstart() does not keep ptmx file descriptor open', function()
+      -- Start another job (using libuv)
+      command('let g:job_opts.pty = 0')
+      local other_jobid = eval("jobstart(['cat', '-'], g:job_opts)")
+      local other_pid = eval('jobpid(' .. other_jobid .. ')')
+
+      -- Other job doesn't block first job from recieving SIGHUP on jobclose()
+      command('call jobclose(j)')
+      -- Have to wait so that the SIGHUP can be processed by tty-test on time.
+      -- Can't wait for the next message in case this test fails, if it fails
+      -- there won't be any more messages, and the test would hang.
+      helpers.sleep(100)
+      local err = exc_exec('call jobpid(j)')
+      eq('Vim(call):E900: Invalid job id', err)
+
+      -- cleanup
+      eq(other_pid, eval('jobpid(' .. other_jobid .. ')'))
+      command('call jobstop(' .. other_jobid .. ')')
     end)
   end)
 end)
@@ -509,8 +666,9 @@ describe("pty process teardown", function()
   end)
 
   it("does not prevent/delay exit. #4798 #4900", function()
+    if helpers.pending_win32(pending) then return end
     -- Use a nested nvim (in :term) to test without --headless.
-    execute(":terminal '"..helpers.nvim_prog
+    feed_command(":terminal '"..helpers.nvim_prog
       -- Use :term again in the _nested_ nvim to get a PTY process.
       -- Use `sleep` to simulate a long-running child of the PTY.
       .."' +terminal +'!(sleep 300 &)' +qa")
