@@ -1,12 +1,12 @@
 local helpers = require('test.functional.helpers')(after_each)
 local Screen = require('test.functional.ui.screen')
 local clear, feed, meths = helpers.clear, helpers.feed, helpers.meths
-local insert, execute = helpers.insert, helpers.execute
+local insert, feed_command = helpers.insert, helpers.feed_command
 local eq, funcs = helpers.eq, helpers.funcs
 
 if helpers.pending_win32(pending) then return end
 
-describe('Mouse input', function()
+describe('ui/mouse/input', function()
   local screen
 
   before_each(function()
@@ -123,9 +123,9 @@ describe('Mouse input', function()
     end)
 
     it('in tabline on filler space moves tab to the end', function()
-      execute('%delete')
+      feed_command('%delete')
       insert('this is foo')
-      execute('silent file foo | tabnew | file bar')
+      feed_command('silent file foo | tabnew | file bar')
       insert('this is bar')
       screen:expect([[
         {tab: + foo }{sel: + bar }{fill:          }{tab:X}|
@@ -153,14 +153,16 @@ describe('Mouse input', function()
     end)
 
     it('in tabline to the left moves tab left', function()
-      if os.getenv("TRAVIS") and helpers.os_name() == "osx" then
-        pending("[Fails on Travis macOS. #4874]", function() end)
+      if helpers.skip_fragile(pending,
+        os.getenv("TRAVIS") and (helpers.os_name() == "osx"
+          or os.getenv("CLANG_SANITIZER") == "ASAN_UBSAN"))  -- #4874
+      then
         return
       end
 
-      execute('%delete')
+      feed_command('%delete')
       insert('this is foo')
-      execute('silent file foo | tabnew | file bar')
+      feed_command('silent file foo | tabnew | file bar')
       insert('this is bar')
       screen:expect([[
         {tab: + foo }{sel: + bar }{fill:          }{tab:X}|
@@ -188,9 +190,9 @@ describe('Mouse input', function()
     end)
 
     it('in tabline to the right moves tab right', function()
-      execute('%delete')
+      feed_command('%delete')
       insert('this is foo')
-      execute('silent file foo | tabnew | file bar')
+      feed_command('silent file foo | tabnew | file bar')
       insert('this is bar')
       screen:expect([[
         {tab: + foo }{sel: + bar }{fill:          }{tab:X}|
@@ -218,9 +220,9 @@ describe('Mouse input', function()
     end)
 
     it('out of tabline under filler space moves tab to the end', function()
-      execute('%delete')
+      feed_command('%delete')
       insert('this is foo')
-      execute('silent file foo | tabnew | file bar')
+      feed_command('silent file foo | tabnew | file bar')
       insert('this is bar')
       screen:expect([[
         {tab: + foo }{sel: + bar }{fill:          }{tab:X}|
@@ -256,14 +258,16 @@ describe('Mouse input', function()
     end)
 
     it('out of tabline to the left moves tab left', function()
-      if os.getenv("TRAVIS") and helpers.os_name() == "osx" then
-        pending("[Fails on Travis macOS. #4874]", function() end)
+      if helpers.skip_fragile(pending,
+        os.getenv("TRAVIS") and (helpers.os_name() == "osx"
+          or os.getenv("CLANG_SANITIZER") == "ASAN_UBSAN"))  -- #4874
+      then
         return
       end
 
-      execute('%delete')
+      feed_command('%delete')
       insert('this is foo')
-      execute('silent file foo | tabnew | file bar')
+      feed_command('silent file foo | tabnew | file bar')
       insert('this is bar')
       screen:expect([[
         {tab: + foo }{sel: + bar }{fill:          }{tab:X}|
@@ -299,9 +303,9 @@ describe('Mouse input', function()
     end)
 
     it('out of tabline to the right moves tab right', function()
-      execute('%delete')
+      feed_command('%delete')
       insert('this is foo')
-      execute('silent file foo | tabnew | file bar')
+      feed_command('silent file foo | tabnew | file bar')
       insert('this is bar')
       screen:expect([[
         {tab: + foo }{sel: + bar }{fill:          }{tab:X}|
@@ -348,9 +352,9 @@ describe('Mouse input', function()
     end)
 
     it('left click in default tabline (position 4) switches to tab', function()
-      execute('%delete')
+      feed_command('%delete')
       insert('this is foo')
-      execute('silent file foo | tabnew | file bar')
+      feed_command('silent file foo | tabnew | file bar')
       insert('this is bar')
       screen:expect([[
         {tab: + foo }{sel: + bar }{fill:          }{tab:X}|
@@ -371,9 +375,9 @@ describe('Mouse input', function()
 
     it('left click in default tabline (position 24) closes tab', function()
       meths.set_option('hidden', true)
-      execute('%delete')
+      feed_command('%delete')
       insert('this is foo')
-      execute('silent file foo | tabnew | file bar')
+      feed_command('silent file foo | tabnew | file bar')
       insert('this is bar')
       screen:expect([[
         {tab: + foo }{sel: + bar }{fill:          }{tab:X}|
@@ -394,9 +398,9 @@ describe('Mouse input', function()
 
     it('double click in default tabline (position 4) opens new tab', function()
       meths.set_option('hidden', true)
-      execute('%delete')
+      feed_command('%delete')
       insert('this is foo')
-      execute('silent file foo | tabnew | file bar')
+      feed_command('silent file foo | tabnew | file bar')
       insert('this is bar')
       screen:expect([[
         {tab: + foo }{sel: + bar }{fill:          }{tab:X}|
@@ -417,13 +421,13 @@ describe('Mouse input', function()
 
     describe('%@ label', function()
       before_each(function()
-        execute([[
+        feed_command([[
           function Test(...)
             let g:reply = a:000
             return copy(a:000)  " Check for memory leaks: return should be freed
           endfunction
         ]])
-        execute([[
+        feed_command([[
           function Test2(...)
             return call('Test', a:000 + [2])
           endfunction
@@ -529,9 +533,9 @@ describe('Mouse input', function()
       fill = { reverse=true },
       vis  = { background=Screen.colors.LightGrey }
     })
-    execute('silent file foo | tabnew | file bar')
+    feed_command('silent file foo | tabnew | file bar')
     insert('this is bar')
-    execute('tabprevious')  -- go to first tab
+    feed_command('tabprevious')  -- go to first tab
     screen:expect([[
       {sel: + foo }{tab: + bar }{fill:          }{tab:X}|
       mouse                    |
@@ -637,7 +641,7 @@ describe('Mouse input', function()
     mouse scrolling
     ]])
     screen:try_resize(53, 14)
-    execute('sp', 'vsp')
+    feed_command('sp', 'vsp')
     screen:expect([[
       lines                     {4:|}lines                     |
       to                        {4:|}to                        |
@@ -654,7 +658,7 @@ describe('Mouse input', function()
       {4:[No Name] [+]                                        }|
       :vsp                                                 |
     ]])
-    feed('<MouseUp><0,0>')
+    feed('<ScrollWheelDown><0,0>')
     screen:expect([[
       mouse scrolling           {4:|}lines                     |
       ^                          {4:|}to                        |
@@ -671,7 +675,7 @@ describe('Mouse input', function()
       {4:[No Name] [+]                                        }|
                                                            |
     ]])
-    feed('<MouseDown><27,0>')
+    feed('<ScrollWheelUp><27,0>')
     screen:expect([[
       mouse scrolling           {4:|}text                      |
       ^                          {4:|}with                      |
@@ -688,7 +692,7 @@ describe('Mouse input', function()
       {4:[No Name] [+]                                        }|
                                                            |
     ]])
-    feed('<MouseDown><27,7><MouseDown>')
+    feed('<ScrollWheelUp><27,7><ScrollWheelUp>')
     screen:expect([[
       mouse scrolling           {4:|}text                      |
       ^                          {4:|}with                      |
@@ -750,12 +754,12 @@ describe('Mouse input', function()
       })
       feed('ggdG')
 
-      execute('set concealcursor=n')
-      execute('set nowrap')
-      execute('syntax match NonText "\\<amet\\>" conceal')
-      execute('syntax match NonText "\\cs\\|g." conceal cchar=X')
-      execute('syntax match NonText "\\%(lo\\|cl\\)." conceal')
-      execute('syntax match NonText "Lo" conceal cchar=Y')
+      feed_command('set concealcursor=n')
+      feed_command('set nowrap')
+      feed_command('syntax match NonText "\\<amet\\>" conceal')
+      feed_command('syntax match NonText "\\cs\\|g." conceal cchar=X')
+      feed_command('syntax match NonText "\\%(lo\\|cl\\)." conceal')
+      feed_command('syntax match NonText "Lo" conceal cchar=Y')
 
       insert([[
       Lorem ipsum dolor sit amet, consetetur sadipscing elitr.
@@ -766,7 +770,7 @@ describe('Mouse input', function()
     end)
 
     it('(level 1) click on non-wrapped lines', function()
-      execute('let &conceallevel=1', 'echo')
+      feed_command('let &conceallevel=1', 'echo')
 
       feed('<esc><LeftMouse><0,0>')
       screen:expect([[
@@ -814,7 +818,7 @@ describe('Mouse input', function()
     end) -- level 1 - non wrapped
 
     it('(level 1) click on wrapped lines', function()
-      execute('let &conceallevel=1', 'let &wrap=1', 'echo')
+      feed_command('let &conceallevel=1', 'let &wrap=1', 'echo')
 
       feed('<esc><LeftMouse><0,0>')
       screen:expect([[
@@ -863,7 +867,7 @@ describe('Mouse input', function()
 
 
     it('(level 2) click on non-wrapped lines', function()
-      execute('let &conceallevel=2', 'echo')
+      feed_command('let &conceallevel=2', 'echo')
 
       feed('<esc><LeftMouse><0,0>')
       screen:expect([[
@@ -911,7 +915,7 @@ describe('Mouse input', function()
     end) -- level 2 - non wrapped
 
     it('(level 2) click on wrapped lines', function()
-      execute('let &conceallevel=2', 'let &wrap=1', 'echo')
+      feed_command('let &conceallevel=2', 'let &wrap=1', 'echo')
 
       feed('<esc><LeftMouse><0,0>')
       screen:expect([[
@@ -960,7 +964,7 @@ describe('Mouse input', function()
 
 
     it('(level 3) click on non-wrapped lines', function()
-      execute('let &conceallevel=3', 'echo')
+      feed_command('let &conceallevel=3', 'echo')
 
       feed('<esc><LeftMouse><0,0>')
       screen:expect([[
@@ -1008,7 +1012,7 @@ describe('Mouse input', function()
     end) -- level 3 - non wrapped
 
     it('(level 3) click on wrapped lines', function()
-      execute('let &conceallevel=3', 'let &wrap=1', 'echo')
+      feed_command('let &conceallevel=3', 'let &wrap=1', 'echo')
 
       feed('<esc><LeftMouse><0,0>')
       screen:expect([[
